#include <OpenDoor.h>
#include <ctype.h>
#include <sqlite3.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

int open_database(sqlite3 **db) {
	static const char *qtable = "CREATE TABLE IF NOT EXISTS questions (id INTEGER PRIMARY KEY, question TEXT, addanswers INTEGER)";
	static const char *atable = "CREATE TABLE IF NOT EXISTS answers (id INTEGER PRIMARY KEY, qid INTEGER, answer TEXT)";
	static const char *rtable = "CREATE TABLE IF NOT EXISTS results (qid INTEGER, aid INTEGER, userhandle TEXT)";
	int rc;
	char *err_msg;
	rc = sqlite3_open("voting.db3", db);
	
	if (rc) {
		return 0;
	}
	
	
	sqlite3_busy_timeout(*db, 5000);
	
	rc = sqlite3_exec(*db, qtable, 0, 0, &err_msg);
    if (rc != SQLITE_OK ) {
        fprintf(stderr, "SQL error: %s", err_msg);
        sqlite3_free(err_msg);
        sqlite3_close(*db);
        return 0;
	}

	rc = sqlite3_exec(*db, atable, 0, 0, &err_msg);
    if (rc != SQLITE_OK ) {
        fprintf(stderr, "SQL error: %s", err_msg);
        sqlite3_free(err_msg);
        sqlite3_close(*db);
        return 0;
	}
	rc = sqlite3_exec(*db, rtable, 0, 0, &err_msg);
    if (rc != SQLITE_OK ) {
        fprintf(stderr, "SQL error: %s", err_msg);
        sqlite3_free(err_msg);
        sqlite3_close(*db);
        return 0;
	}
	return 1;
}

struct answer_t {
	int aid;
	char *answer;
};

void add_question() {
	char buffer[256];
	
	char *question;
	
	char **answers;
	
	int answer_count = 0;
	char c;
	int addownanswers = 0;
	int i;
	int qid;
	sqlite3 *db;
	sqlite3_stmt *res;
	int rc;
	
	static const char *sql = "INSERT INTO questions (question, addanswers) VALUES(?, ?)";
	static const char *sql2 = "INSERT INTO answers (qid, answer) VALUES(?, ?)";
	
	
	od_clr_scr();
	
	od_printf("`bright white`Your Question: `white`");
	od_input_str(buffer, 256, 32, 126);
	
	if (strlen(buffer) == 0) {
		return;
	}
	
	question = strdup(buffer);
	
	od_printf("\r\n\r\n`bright white`Please enter your answers (Enter a blank line when finished!)\r\n`white`");
	
	while (1) {
		od_printf("\r\n`bright green`Answer `bright magenta`%d`bright white`: `white`", answer_count + 1);
		od_input_str(buffer, 256, 32, 126);

		if (strlen(buffer) == 0) {
			break;
		}
		if (answer_count == 0) {
			answers = (char **)malloc(sizeof(char *));
		} else {
			answers = (char **)realloc(answers, sizeof(char *) * (answer_count + 1));
		}
		
		answers[answer_count] = strdup(buffer);
		answer_count++;
	}
	
	od_printf("\r\n\r\n`bright white`May voters add their own answer (Y/N) ? `white`");
	c = od_get_answer("YyNn");
	if (tolower(c) == 'y') {
		addownanswers = 1;
	}
	
	od_printf("\r\n\r\n`bright white`Save this question? (Y/N) ? `white`");
	
	c = od_get_answer("YyNn");
	if (tolower(c) == 'y') {
		if (!open_database(&db)) {
			od_printf("`bright red`Error opening database! Please notify Sysop`white`\r\n");
			goto cleanup2;
		}
		
		rc = sqlite3_prepare_v2(db, sql, strlen(sql), &res, NULL);
		if (rc != SQLITE_OK) {
			od_printf("`bright red`Error preparing sql! Please notify Sysop`white` No1\r\n");
			sqlite3_close(db);
			goto cleanup2;
		}
		
		sqlite3_bind_text(res, 1, question, -1, 0);
		sqlite3_bind_int(res, 2, addownanswers);
		
		sqlite3_step(res);
		
		qid = sqlite3_last_insert_rowid(db);
		
		sqlite3_finalize(res);
		
		for (i=0;i<answer_count;i++) {
			rc = sqlite3_prepare_v2(db, sql2, strlen(sql2), &res, NULL);
			if (rc != SQLITE_OK) {
				od_printf("`bright red`Error preparing sql! Please notify Sysop`white`\r\n");
				sqlite3_close(db);
				goto cleanup2;
			}
			
			sqlite3_bind_int(res, 1, qid);
			sqlite3_bind_text(res, 2, answers[i], -1, 0);
			
			sqlite3_step(res);
			sqlite3_finalize(res);
		}
		sqlite3_close(db);
	}

cleanup2:

	for (i=0;i<answer_count;i++) {
		free(answers[i]);
	}
	free(answers);
	free(question);
}

void view_results(int qid) {
	static const char *sql = "SELECT COUNT(*) FROM results WHERE qid = ?";
	static const char *sql2 = "SELECT COUNT(*) FROM results WHERE qid = ? AND aid = ?";
	static const char *sql3 = "SELECT id, answer FROM answers WHERE qid = ?";
	static const char *sql4 = "SELECT question FROM questions WHERE id = ?";
	sqlite3 *db;
	
	if (!open_database(&db)) {
		od_printf("`bright red`Error opening database! Please notify Sysop`white`\r\n");
		return;
	}
	sqlite3_stmt *res;
	int rc;
	char *question;	
	int total_answers;
	int i;
	struct answer_t **answers;
	int answer_count = 0;
		
	rc = sqlite3_prepare_v2(db, sql4, strlen(sql4), &res, NULL);
	if (rc != SQLITE_OK) {
		od_printf("`bright red`Error preparing sql! Please notify Sysop`white`\r\n");
		sqlite3_close(db);
		return;		
	}
	sqlite3_bind_int(res, 1, qid);	
	
	if (sqlite3_step(res) != SQLITE_ROW) {
		sqlite3_finalize(res);
		sqlite3_close(db);
		
		return;
	}
	
	question = strdup(sqlite3_column_text(res, 0));
	
	sqlite3_finalize(res);

	rc = sqlite3_prepare_v2(db, sql, strlen(sql), &res, NULL);
	if (rc != SQLITE_OK) {
		od_printf("`bright red`Error preparing sql! Please notify Sysop`white`\r\n");
		sqlite3_close(db);
		free(question);
		return;
	}
	sqlite3_bind_int(res, 1, qid);
	
	if (sqlite3_step(res) != SQLITE_ROW) {
		od_printf("`bright red`Error preparing sql! Please notify Sysop`white`\r\n");
		sqlite3_finalize(res);
		sqlite3_close(db);
		free(question);
		return;
	}
	
	total_answers = sqlite3_column_int(res, 0);
	sqlite3_finalize(res);
	
	rc = sqlite3_prepare_v2(db, sql3, strlen(sql3), &res, NULL);
	if (rc != SQLITE_OK) {
		od_printf("`bright red`Error preparing sql! Please notify Sysop`white`\r\n");
		sqlite3_close(db);
		free(question);
		return;		
	}
	sqlite3_bind_int(res, 1, qid);
	
	while (sqlite3_step(res) == SQLITE_ROW) {
		if (answer_count == 0) {
			answers = (struct answer_t **)malloc(sizeof(struct answer_t *));
		} else {
			answers = (struct answer_t **)realloc(answers, sizeof(struct answer_t *) * (answer_count + 1));
		}
		
		answers[answer_count] = (struct answer_t *)malloc(sizeof(struct answer_t));
		
		answers[answer_count]->aid = sqlite3_column_int(res, 0);
		answers[answer_count]->answer = strdup(sqlite3_column_text(res, 1));
		answer_count++;		
	}
	sqlite3_finalize(res);
	
	od_clr_scr();
	od_printf("`bright green`Question:`white`\r\n\r\n");
	od_printf("`bright white`%s`white`\r\n\r\n", question);
	od_printf("`bright blue`Answers:`white`\r\n\r\n");

	
	for (i=0;i<answer_count;i++) {
		rc = sqlite3_prepare_v2(db, sql2, strlen(sql2), &res, NULL);
		if (rc != SQLITE_OK) {
			continue;
		}
		
		sqlite3_bind_int(res, 1, qid);
		sqlite3_bind_int(res, 2, answers[i]->aid);
		if (sqlite3_step(res) != SQLITE_ROW) {
			sqlite3_finalize(res);
			continue;
		}
		
		od_printf("`bright cyan`%d. `bright white`%s (`bright magenta`%d`bright white`)`white`\r\n", i + 1, answers[i]->answer, sqlite3_column_int(res, 0));
		sqlite3_finalize(res);
	}
	
	sqlite3_close(db);
	
	od_printf("\r\n`bright magenta`A total of %d votes were cast!\r\n\r\n`white`", total_answers);
	
	for (i=0;i<answer_count;i++) {
		free(answers[i]->answer);
		free(answers[i]);
	}
	free(answers);
	free(question);	
}

void vote_on_this_question(int qid) {
	static const char *sql = "SELECT question,addanswers FROM questions WHERE id = ?";
	static const char *sql2 = "SELECT aid FROM results WHERE qid = ? AND userhandle = ?";
	static const char *sql3 = "SELECT id, answer FROM answers WHERE qid = ?";
	static const char *sql4 = "INSERT INTO results (qid, aid, userhandle) VALUES(?, ?, ?)";
	static const char *sql5 = "INSERT INTO answers (qid, answer) VALUES(?, ?)";
	char buffer[11];
	char newanswer[256];
	sqlite3 *db;
	sqlite3_stmt *res;
	int rc;
	char *question;
	int newanswersallowed = 0;
	int aid;
	int done = 0;
	struct answer_t **answers;
	int answer_count = 0;
	int i;

	
	if (!open_database(&db)) {
		od_printf("`bright red`Error opening database! Please notify Sysop`white`\r\n");
		return;
	}

	rc = sqlite3_prepare_v2(db, sql, strlen(sql), &res, NULL);
	if (rc != SQLITE_OK) {
		od_printf("`bright red`Error preparing sql! Please notify Sysop`white`\r\n");
		sqlite3_close(db);
		return;		
	}
	sqlite3_bind_int(res, 1, qid);	
	
	if (sqlite3_step(res) != SQLITE_ROW) {
		sqlite3_finalize(res);
		sqlite3_close(db);
		
		return;
	}
	
	question = strdup(sqlite3_column_text(res, 0));
	newanswersallowed = sqlite3_column_int(res, 1);
	
	sqlite3_finalize(res);

	rc = sqlite3_prepare_v2(db, sql2, strlen(sql2), &res, NULL);
	sqlite3_bind_int(res, 1, qid);
	sqlite3_bind_text(res, 2, (od_control_get()->user_name == NULL ? od_control_get()->user_handle : od_control_get()->user_name), -1, 0);
	
	if (rc != SQLITE_OK) {
		od_printf("`bright red`Error preparing sql! Please notify Sysop`white`\r\n");
		sqlite3_close(db);
		free(question);
		return;		
	}
	
	if (sqlite3_step(res) == SQLITE_ROW) {
		od_printf("`bright yellow`You've already voted on this question!`white`\r\n");
		sqlite3_finalize(res);
		sqlite3_close(db);
		free(question);
		return;
	}
	sqlite3_finalize(res);
	
	rc = sqlite3_prepare_v2(db, sql3, strlen(sql3), &res, NULL);
	if (rc != SQLITE_OK) {
		od_printf("`bright red`Error preparing sql! Please notify Sysop`white`\r\n");
		sqlite3_close(db);
		free(question);
		return;		
	}
	sqlite3_bind_int(res, 1, qid);
	
	while (sqlite3_step(res) == SQLITE_ROW) {
		if (answer_count == 0) {
			answers = (struct answer_t **)malloc(sizeof(struct answer_t *));
		} else {
			answers = (struct answer_t **)realloc(answers, sizeof(struct answer_t *) * (answer_count + 1));
		}
		
		answers[answer_count] = (struct answer_t *)malloc(sizeof(struct answer_t));
		
		answers[answer_count]->aid = sqlite3_column_int(res, 0);
		answers[answer_count]->answer = strdup(sqlite3_column_text(res, 1));
		answer_count++;		
	}
	sqlite3_finalize(res);

	while (!done) {
		od_clr_scr();
		od_printf("`bright green`Question:`white`\r\n\r\n");
		od_printf("`bright white`%s`white`\r\n\r\n", question);
		od_printf("`bright blue`Answers:`white`\r\n\r\n");
		
		for (i=0;i<answer_count;i++) {
			od_printf("`bright cyan`%d. `bright white`%s\r\n", i + 1, answers[i]->answer);
		}
		if (newanswersallowed) {
			od_printf("`bright cyan`0. `bright white`Add your own answer...\r\n");
		}
		od_printf("\r\n:> ");
		
		od_input_str(buffer, 11, '0', '9');
		aid = atoi(buffer);
		
		if (aid == 0) {
			if (newanswersallowed) {
				od_printf("\r\n\r\nYour answer: ");
				od_input_str(newanswer, 256, 32, 126);
				
				rc = sqlite3_prepare_v2(db, sql5, strlen(sql5), &res, NULL);
				if (rc != SQLITE_OK) {
					od_printf("`bright red`Error preparing sql! Please notify Sysop`white`\r\n");
					sqlite3_close(db);				
					goto cleanup;
				}	
				sqlite3_bind_int(res, 1, qid);
				sqlite3_bind_text(res, 2, newanswer, -1, 0);
				
				if (sqlite3_step(res) != SQLITE_DONE) {
					od_printf("`bright red`Error adding answer! Please notify Sysop`white`\r\n");
					sqlite3_close(db);				
					goto cleanup;
				}
				aid = sqlite3_last_insert_rowid(db);
				sqlite3_finalize(res);
				rc = sqlite3_prepare_v2(db, sql4, strlen(sql4), &res, NULL);
				if (rc != SQLITE_OK) {
					od_printf("`bright red`Error preparing sql! Please notify Sysop`white`\r\n");
					sqlite3_close(db);				
					goto cleanup;
				}
				sqlite3_bind_int(res, 1, qid);
				sqlite3_bind_int(res, 2, aid);
				sqlite3_bind_text(res, 3, (od_control_get()->user_name == NULL ? od_control_get()->user_handle : od_control_get()->user_name), -1, 0);
				
				sqlite3_step(res);
				sqlite3_finalize(res);
				done = 1;
			} else {
				continue;
			}
		} else {
			aid--;
			
			if (aid >= 0 && aid < answer_count) {
				
				rc = sqlite3_prepare_v2(db, sql4, strlen(sql4), &res, NULL);
				if (rc != SQLITE_OK) {
					od_printf("`bright red`Error preparing sql! Please notify Sysop`white`\r\n");
					sqlite3_close(db);				
					goto cleanup;
				}
				sqlite3_bind_int(res, 1, qid);
				sqlite3_bind_int(res, 2, answers[aid]->aid);
				sqlite3_bind_text(res, 3, (od_control_get()->user_name == NULL ? od_control_get()->user_handle : od_control_get()->user_name), -1, 0);
				
				sqlite3_step(res);
				sqlite3_finalize(res);
				
				done = 1;
			}
		}
	}
	
	sqlite3_close(db);
	
cleanup:

	for (i=0;i<answer_count;i++) {
		free(answers[i]->answer);
		free(answers[i]);
	}
	free(answers);
	free(question);
}

struct question_t {
	int qid;
	char *question;
};


void list_questions(int action) {
	// list questions
	static const char *sql = "SELECT id,question FROM questions";
	
	sqlite3 *db;
	sqlite3_stmt *res;
	char buffer[12];
	int qid;
	int rc;
	int i;
	struct question_t **questions;
	int question_count = 0;
	
	if (!open_database(&db)) {
		od_printf("`bright red`Error opening database! Please notify Sysop`white`\r\n");
		return;
	}
	
	rc = sqlite3_prepare_v2(db, sql, strlen(sql), &res, NULL);
	if (rc != SQLITE_OK) {
		od_printf("`bright red`Error preparing sql! Please notify Sysop`white`\r\n");
		sqlite3_close(db);
		return;
	}
	
	while (sqlite3_step(res) == SQLITE_ROW) {
		if (question_count == 0) {
			questions = (struct question_t **)malloc(sizeof(struct question_t *));
		} else {
			questions = (struct question_t **)realloc(questions, sizeof(struct question_t *) * (question_count + 1));
		}
		
		questions[question_count] = (struct question_t *)malloc(sizeof(struct question_t));
		
		questions[question_count]->qid = sqlite3_column_int(res, 0);
		questions[question_count]->question = strdup(sqlite3_column_text(res, 1));
		question_count++;
	}
	
	
	sqlite3_finalize(res);
	sqlite3_close(db);

	if (question_count > 0) {
		od_clr_scr();
		od_printf("`bright cyan`Questions, questions, questions...\r\n\r\n`white`");
		for (i=0;i<question_count;i++) {
			od_printf("`bright magenta`%d`bright white`. %s\r\n`white`", i+1, questions[i]->question);
		}
		od_input_str(buffer, 11, '0', '9');
		qid = atoi(buffer);
		qid--;
		
		if (qid >= 0 && qid < question_count) {
			if (action == 1) {
				vote_on_this_question(questions[qid]->qid);
			} else if (action == 2) {
				view_results(questions[qid]->qid);
			}
		}		
	} else {
		od_printf("`bright yellow`No questions!`white`\r\n");
		od_printf("`bright white`Press a key...`white`\r\n");
		od_get_key(TRUE);
		return;
	}

	for (i = 0; i< question_count;i++) {
		free(questions[i]->question);
		free(questions[i]);
	}
	free(questions);
	od_printf("`bright white`Press a key...`white`\r\n");
	od_get_key(TRUE);	
}

#if _MSC_VER
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,LPSTR lpszCmdLine,int nCmdShow)
{
#else
int main(int argc, char **argv) 
{
#endif
	int done = 0;
	char c;
	
#if _MSC_VER
	od_parse_cmd_line(lpszCmdLine);
#else
	od_parse_cmd_line(argc, argv);
#endif
	
	while(!done) {
		od_clr_scr();
		od_send_file("votemenu.ans");
		
		c = od_get_answer("VvRrAaQq");
		
		switch(tolower(c)) {
			case 'v':
				list_questions(1);
				break;
			case 'r':
				list_questions(2);
				break;
			case 'a':
				add_question();
				break;
			case 'q':
				done = 1;
				break;
		}
	}
	od_exit(0, FALSE);
}
