#include <ctype.h>
#include <sqlite3.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

int open_database(sqlite3 **db) {
	static const char *qtable = "CREATE TABLE IF NOT EXISTS questions (id INTEGER PRIMARY KEY, question TEXT, addanswers INTEGER)";
	static const char *atable = "CREATE TABLE IF NOT EXISTS answers (id INTEGER PRIMARY KEY, qid INTEGER, answer TEXT)";
	static const char *rtable = "CREATE TABLE IF NOT EXISTS results (qid INTEGER, aid INTEGER, userhandle TEXT)";
	int rc;
	char *err_msg;
	rc = sqlite3_open("voting.db3", db);
	
	if (rc) {
		return 0;
	}
	
	
	sqlite3_busy_timeout(*db, 5000);
	
	rc = sqlite3_exec(*db, qtable, 0, 0, &err_msg);
    if (rc != SQLITE_OK ) {
        fprintf(stderr, "SQL error: %s", err_msg);
        sqlite3_free(err_msg);
        sqlite3_close(*db);
        return 0;
	}

	rc = sqlite3_exec(*db, atable, 0, 0, &err_msg);
    if (rc != SQLITE_OK ) {
        fprintf(stderr, "SQL error: %s", err_msg);
        sqlite3_free(err_msg);
        sqlite3_close(*db);
        return 0;
	}
	rc = sqlite3_exec(*db, rtable, 0, 0, &err_msg);
    if (rc != SQLITE_OK ) {
        fprintf(stderr, "SQL error: %s", err_msg);
        sqlite3_free(err_msg);
        sqlite3_close(*db);
        return 0;
	}
	return 1;
}

struct question_t {
	int qid;
	char *question;
};

void delete_question(int qid) {
	static const char *sql1 = "DELETE FROM questions WHERE id = ?";
	static const char *sql2 = "DELETE FROM answers WHERE qid = ?";
	static const char *sql3 = "DELETE FROM results WHERE qid = ?";
	
	sqlite3 *db;
	sqlite3_stmt *res;
	
	if (!open_database(&db)) {
		fprintf(stderr, "Error opening database!\n");
		return;
	}
	
	if (sqlite3_prepare_v2(db, sql1, strlen(sql1), &res, NULL) != SQLITE_OK) {
		fprintf(stderr, "Error preparing SQL\n");
		sqlite3_close(db);
		return;
	}
	sqlite3_bind_int(res, 1, qid);
	sqlite3_step(res);
	sqlite3_finalize(res);
	
	if (sqlite3_prepare_v2(db, sql2, strlen(sql2), &res, NULL) != SQLITE_OK) {
		fprintf(stderr, "Error preparing SQL\n");
		sqlite3_close(db);
		return;
	}
	sqlite3_bind_int(res, 1, qid);
	sqlite3_step(res);
	sqlite3_finalize(res);	

	if (sqlite3_prepare_v2(db, sql3, strlen(sql3), &res, NULL) != SQLITE_OK) {
		fprintf(stderr, "Error preparing SQL\n");
		sqlite3_close(db);
		return;
	}
	sqlite3_bind_int(res, 1, qid);
	sqlite3_step(res);
	sqlite3_finalize(res);	
	
	printf("Question deleted!\n");
	
	sqlite3_close(db);
}

int main(int argc, char **argv) {
	sqlite3 *db;
	sqlite3_stmt *res;
	struct question_t **questions;
	int question_count = 0;
	int i;
	static const char *sql = "SELECT id,question FROM questions";
	int qid;
	if (!open_database(&db)) {
		fprintf(stderr, "Error opening database!\n");
		return -1;
	}
	
	if (sqlite3_prepare_v2(db, sql, strlen(sql), &res, NULL) != SQLITE_OK) {
		fprintf(stderr, "Error preparing SQL\n");
		sqlite3_close(db);
		return -1;
	}
	
	while (sqlite3_step(res) == SQLITE_ROW) {
		if (question_count == 0) {
			questions = (struct question_t **)malloc(sizeof(struct question_t *));
		} else {
			questions = (struct question_t **)realloc(questions, sizeof(struct question_t *) * (question_count + 1));
		}
		
		questions[question_count] = (struct question_t *)malloc(sizeof(struct question_t));
		
		questions[question_count]->qid = sqlite3_column_int(res, 0);
		questions[question_count]->question = strdup(sqlite3_column_text(res, 1));
		question_count++;
	}
	
	
	sqlite3_finalize(res);
	sqlite3_close(db);
	
	if (question_count == 0) {
		printf("No questions.\n");
		return 0;
	}
	
	for (i = 0; i< 	question_count; i++) {
		printf("%d. %s\n", i + 1, questions[i]->question);
	}
	
	printf("\nDelete Which ? ");
	
	scanf("%d", &qid);
	
	if (qid == 0) {
		return 0;
	}
	
	if (qid > 0 && qid <= question_count) {
		delete_question(questions[qid-1]->qid);
	} else {
		printf("\nInvalid question\n");
	}
	return 0;
}
